locklost: aLIGO IFO lock loss tracking and analysis tool
========================================================

This package provides a set of tools for analyzing LIGO detector "lock
losses".  It consists of four main components:

* `search` for detector lock losses based on guardian state
  transitions.
* `analyze` lock losses to generate plots and look for identifying
  features.
* `online` analysis to monitor for lock losses and automatically run
  follow-up analyses.
* `web` interface to view lock loses event pages.

A command line interface is provided to launch the `online` analysis
and condor jobs to `search` for and `analyze` events.


# Deployments

Each site (LHO and LLO) has a dedicated "lockloss" account on the site
LDAS cluster where the primary lock loss tracking via `locklost` takes
place:

* https://ldas-jobs.ligo-wa.caltech.edu/~lockloss/
* https://ldas-jobs.ligo-la.caltech.edu/~lockloss/

These accounts have deployments of the `locklost` package and command
line interface, run the online and followup condor jobs, and host the
web pages.

## deploying new versions

When a new tag has been applied to the code, the following can be run
to deploy the new code at the site:
```shell
$ ssh lockloss@detchar.ligo-la.caltech.edu
$ cd src/locklost
$ git pull
$ ./test/run
$ locklost-deploy
```
If there are changes to the online search, restart the online condor
process:
```shell
$ locklost online restart
```


# Usage

To run the online analysis:
```shell
$ locklost online start
```
This launches a condor job that runs the online analysis.

To launch a condor job to search for lock losses within some time
window:
```shell
$ locklost search GPS_START GPS_END
```
This will find lock losses with the specified time range, but will not
run the follow-up analyses.

To run an analysis on a specific lock loss time found from the search
above:
```shell
$ locklost analyze GPS_TIME
```
To launch a condor job to analyze all events within a time range:
```shell
$ locklost analyze --dag GPS_START GPS_END
```
To re-analyze events add the `--rerun` flag:
```shell
$ locklost analyze --dag GPS_START GPS_END --rerun
```


# Analysis plugins

Lock loss event analyses are handled by a set of "plugins" located in
the /locklost/plugins/ package directory:

* [`locklost/plugins/`](/locklost/plugins/)

Each module is registered in [`locklost/plugins/__init__.py`](/locklost/plugins/__init__.py).  The
currently enabled follow-up plugins are:

* `discover.discover_data` wait for data to be available
* `refine.refine_event` refine event time
* `saturations.find_saturations` find saturating channels before event
* `lpy.find_lpy` find length/pitch/yaw oscillations in the suspensions
* `glitch.analyze_glitches` look for glitches around event
* `overflows.find_overflows` look for ADC overflows
* `state_start.find_lock_start` find the start of the lock

Each plugin does it's own analysis, although some depend on the output
of other analyses.  The output from any analysis (e.g. plots or data)
should be written into the event directory.


# Developing and contributing

We welcome any help developing further analysis followups,
improvements to the web page, etc.  Please contact
jameson.rollins@ligo.org to coordinate development.

Since the analysis runs best on the site LDAS clusters, you should
contact your local sysadmin about getting access to the local cluster.

Contributions to `locklost` are handled via [GitLab merge
request](https://docs.gitlab.com/ee/user/project/merge_requests/) (see
below).  It's therefore advised that you
[fork](https://docs.gitlab.com/ee/gitlab-basics/fork-project.html)
your own copy the project to work from:

* https://git.ligo.org/jameson.rollins/locklost/forks/new


## install

To start contributing, it's best if you first install your own
personal copy of `locklost` in your personal LDAS home directory.
This will allow you to develop and test changes without disturbing the
production run.

Start by
[cloning](https://docs.gitlab.com/ee/gitlab-basics/command-line-commands.html)
your copy of the project into your LDAS cluster home directory.
Assuming your gitlab account name is "albert.einstein" you can clone
your copy of the project with git like so:
```shell
$ git clone git@git.ligo.org:albert.einstein/locklost.git
```
Note that it's best to access gitlab via [ssh
keys](https://docs.gitlab.com/ee/ssh/), since that will allow you to
easily push changes back to be merged into the main project.

You can install the code into a separate directory using the included
`setup.py`:
```shell
$ python setup.py install --prefix=~/my/path/to/install/
```
It's also possible to run directly out of the source.  Just change
directory into the `locklost` directory that you git cloned, and run
the package directly:
```shell
$ python -m locklost --help
```


## environment variables

`locklost` looks for two environment variables that specify important
configuration parameters:
* `IFO` the interferometer in question (e.g. 'H1' or 'L1')
* `LOCKLOST_EVENT_ROOT` root directory for where events are stored

These variables  can either be specified  in e.g. your `.bashrc`, or
when directly when you run the command line interface:
```shell
$ IFO=L1 LOCKLOST_EVENT_ROOT=~/my/event/dir python -m locklost --help
```

If you're just making web changes (i.e. not attempting to write new
event info) you can just point to  the main lockloss event root
located at `~lockloss/events` at both sites:
```shell
$ IFO=L1 LOCKLOST_EVENT_ROOT=~lockloss/events python -m locklost --help
```


## contributing

All contributions are handled via [GitLab merge
request](https://docs.gitlab.com/ee/user/project/merge_requests/).
The basic procedure for a merge request is as follows:

1. Create a new branch in your local checkout for your proposed changes
```shell
$ git checkout -B my-dev-branch
```

2. Commit all of your changes to this new branch:
```shell
$ git commit ....
```

3. When you've got all changes ready to go, push your development
   branch to your fork of the project (assuming your cloned from your
   fork of the project):
```shell
$ git push origin my-dev-branch
```

4. Navigate to your gitlab page for the project and click the "New
   merge request" button.

5. Specify that you would like to submit a merge request for your new
   development branch
   (e.g. source="albert.einstein/locklost:my-dev-branch") to be merged
   with the upstream master branch
   (e.g. target="jameson.rollins/locklost:master").

6. Watch the merge request page for review of your request.
